from __future__ import annotations

from typing import Optional

from requests import HTTPError

from anylearn.sdk.artifacts.artifact import Artifact
from anylearn.sdk.auth import configure_auth
from anylearn.sdk.client import get_with_token, post_with_token
from anylearn.sdk.context import get_base_url
from anylearn.sdk.errors import (
    AnylearnArtifactDuplicationError,
    AnylearnInvalidResponseError,
)


class ModelArtifact(Artifact):

    @classmethod
    def from_full_name(cls, full_name: str) -> ModelArtifact:
        res = get_with_token(
            f"{get_base_url()}/model/query",
            params={'fullname': full_name},
        )
        if not res or not isinstance(res, list):
            raise AnylearnInvalidResponseError(
                f"Failed to get model {full_name} from server."
            )
        return ModelArtifact(**res[0])

    @classmethod
    def create(
        cls,
        name: str,
        description: Optional[str] = None,
        public: bool = False,
    ) -> ModelArtifact:
        auth = configure_auth()
        # Check if model already exists
        full_name = f"{auth.username}/{name}"
        try:
            get_with_token(
                f"{get_base_url()}/model/query",
                params={'fullname': full_name},
            )
            raise AnylearnArtifactDuplicationError(
                f"Model {full_name} already exists."
            )
        except HTTPError as e:
            if e.response.status_code != 404:
                raise
        # Create model
        res = post_with_token(
            f"{get_base_url()}/model/add",
            data={
                'name': name,
                'description': description,
                'public': 1 if public else 0,
            },
        )
        if not res or not isinstance(res, dict) or not res.get('data'):
            raise AnylearnInvalidResponseError(
                f"Failed to create model {full_name} on server."
            )
        return cls.from_full_name(full_name)



import os
import subprocess
from pathlib import Path
from subprocess import CalledProcessError
from typing import Optional, Union

import paramiko
from rich import print

from anylearn.sdk.errors import AnylearnError
from anylearn.sdk.jumps.channel import (
    JumpsChannel,
    get_jumps_ssh_key_paths,
)


class JumpsUploader:

    def __init__(
        self,
        channel: JumpsChannel,
        local_path: Optional[Union[os.PathLike, bytes, str]],
        compress: bool = True,
    ) -> None:
        self.channel = channel
        self.local_path = Path(local_path)
        if not self.local_path.exists():
            raise AnylearnError(
                f"Local path {self.local_path} does not exist"
            )
        if not self.local_path.is_file() and not self.local_path.is_dir():
            raise AnylearnError(
                f"Local path {self.local_path} is "
                "neither a file nor a directory"
            )
        self.compress = compress
        self._check_channel()
        self.ssh_private_key_path = get_jumps_ssh_key_paths()[0]

    def _check_channel(self):
        if self.channel is None:
            raise AnylearnError(
                "Channel must not be None"
            )
        if any([
            self.channel.id is None,
            self.channel.ip is None,
            self.channel.port is None,
            self.channel.username is None,
            self.channel.path is None,
        ]):
            raise AnylearnError(
                f"Channel is not properly configured: {self.channel}"
            )

    def rsync_available(self):
        try:
            subprocess.check_output(["rsync", "--version"])
            return True
        except:
            return False

    def compression_effective(self):
        # TODO: automatically detect whether compression is effective
        return True

    def upload(self):
        if self.rsync_available():
            return self._upload_by_rsync()
        else:
            with self._open_ssh() as ssh:
                with ssh.open_sftp() as sftp:
                    return self._upload_by_sftp(sftp)

    def _upload_by_rsync(self) -> int:
        if not self.rsync_available():
            raise AnylearnError(
                "Cannot upload by rsync since rsync is not available"
            )
        rsync_options = "-avP"
        if self.compress and self.compression_effective():
            rsync_options += "z"
        ssh_command = (
            "ssh"
            f" -p {self.channel.port}"
            f" -i {self.ssh_private_key_path}"
        )
        rsync_source = (
            f"{str(self.local_path)}/"
            if self.local_path.is_dir()
            else f"{str(self.local_path)}"
        )
        rsync_dest = (
            f"{self.channel.username}@{self.channel.ip}"
            ":"
            f"{self.channel.path}"
        )
        rsync_command = (
            "rsync"
            f" {rsync_options}"
            f" -e \"{ssh_command}\""
            f" {rsync_source}"
            f" {rsync_dest}"
        )
        proc = subprocess.run(rsync_command, shell=True, check=True)
        return proc.returncode

    def _open_ssh(self) -> paramiko.SSHClient:
        self._check_channel()
        compress = self.compress and self.compression_effective()
        ssh = paramiko.SSHClient()
        ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
        ssh.connect(
            hostname=self.channel.ip,
            port=self.channel.port,
            username=self.channel.username,
            key_filename=str(self.ssh_private_key_path),
            compress=compress,
        )
        return ssh

    def _upload_by_sftp(self, sftp: paramiko.SFTPClient) -> int:
        if self.local_path.is_file():
            return self._upload_file_by_sftp(sftp)
        elif self.local_path.is_dir():
            return self._upload_dir_by_sftp(sftp)
        else:
            raise AnylearnError(
                f"Local path {self.local_path} is "
                "neither a file nor a directory"
            )

    def _upload_file_by_sftp(self, sftp: paramiko.SFTPClient) -> int:
        remote_f = Path(self.channel.path) / self.local_path.name
        sftp.put(str(self.local_path), str(remote_f))
        # Preserve file metadata (access time and modification time)
        stat = self.local_path.stat()
        sftp.utime(str(remote_f), (stat.st_atime, stat.st_mtime))
        return 0

    def _upload_dir_by_sftp(self, sftp: paramiko.SFTPClient) -> int:
        # Progress counter
        transfered_size = 0
        transfered_nb_files = 0
        # Traverse local path
        for root, dirs, files in os.walk(
            str(self.local_path),
            topdown=True,
            followlinks=False,
        ):
            local_current_path = Path(root)
            local_relative_path = local_current_path.relative_to(self.local_path)
            remote_current_path = Path(self.channel.path) / local_relative_path
            # Create sub-directories
            for d in dirs:
                sftp.mkdir(str(remote_current_path / d))
            # Transfer files
            for f in files:
                local_f = local_current_path / f
                remote_f = remote_current_path / f
                sftp.put(str(local_f), str(remote_f))
                # Preserve file metadata (access time and modification time)
                stat = local_f.stat()
                sftp.utime(str(remote_f), (stat.st_atime, stat.st_mtime))
                # Counter
                transfered_size += stat.st_size
                transfered_nb_files += 1
                # TODO: display progress with total counts
                print(f"Uploaded {local_f}")
                print(f"  Transfered {transfered_nb_files} files, {transfered_size} bytes")
        return 0



from .artifacts.algorithm import AlgorithmArtifact
from .artifacts.artifact import Artifact, ArtifactState
from .artifacts.dataset import DatasetArtifact
from .artifacts.file import FileArtifact
from .artifacts.model import ModelArtifact
from .auth import Auth
from .context import Context, init
from .project import Project
from .task import Task

def get_task_output(task_id: str) -> FileArtifact:
    task = Task.from_id(task_id)
    return FileArtifact.from_id(task.output_artifact_id)



import os
from pathlib import Path
from typing import Optional, Union

import requests
from urllib3.util.retry import Retry
from requests import HTTPError
from requests.adapters import HTTPAdapter

from anylearn.sdk.errors import AnylearnError


DEFAULT_ANYLEARN_HOST = "https://anylearn.nelbds.cn"


def get_config_path() -> Path:
    config_dir = Path.home() / ".anylearn"
    config_dir.mkdir(parents=True, exist_ok=True)
    return config_dir / "config.yaml"


def get_artifact_download_path() -> Path:
    download_dir = Path.home() / ".anylearn" / "artifacts"
    download_dir.mkdir(parents=True, exist_ok=True)
    return download_dir


def get_tmp_path() -> Path:
    tmp_dir = Path.home() / ".anylearn" / "tmp"
    tmp_dir.mkdir(parents=True, exist_ok=True)
    return tmp_dir


def ensure_dir(dir_: Optional[Union[os.PathLike, bytes, str]]):
    root_dir = Path(dir_)
    if root_dir.exists() and root_dir.is_file():
        raise AnylearnError(
            f"Path `{str(dir_)}` is expected to be a directory, "
            "a file given"
        )
    root_dir.mkdir(parents=True, exist_ok=True)


def raise_for_status(res: requests.Response):
    http_error_msg = ''
    if isinstance(res.reason, bytes):
        try:
            reason = res.reason.decode('utf-8')
        except UnicodeDecodeError:
            reason = res.reason.decode('iso-8859-1')
    else:
        reason = res.reason
    
    if isinstance(res.content, bytes):
        try:
            content = res.content.decode('utf-8')
        except UnicodeDecodeError:
            content = res.content.decode('iso-8859-1')
    else:
        content = res.content

    if 400 <= res.status_code < 500:
        http_error_msg = u'%s: "%s" for url: %s' % (res.status_code, content, res.url)

    elif 500 <= res.status_code < 600:
        http_error_msg = u'%s: "%s" for url: %s' % (res.status_code, reason, res.url)

    if http_error_msg:
        raise HTTPError(http_error_msg, response=res)


class RequestRetrySession(object):
    def __init__(
        self,
        retries=3,
        backoff_factor=1,
        status_forcelist=[429, 500, 502, 503, 504],
        allowed_methods=["HEAD", "GET", "POST", "PUT", "PATCH", "DELETE", "OPTIONS", "TRACE"],
        session=None,
    ):
        self.session = session or requests.session()
        retry = Retry(
            total=retries,
            read=retries,
            connect=retries,
            backoff_factor=backoff_factor,
            status_forcelist=status_forcelist,
            allowed_methods=allowed_methods,
            raise_on_status=False,
        )
        adapter = HTTPAdapter(max_retries=retry)
        self.session.mount('http://', adapter)
        self.session.mount('https://', adapter)

    def __enter__(self):
        return self.session

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.session.close()



import requests
from requests.exceptions import HTTPError

from anylearn.sdk.auth import configure_auth
from anylearn.sdk.utils import RequestRetrySession, raise_for_status


def request(method, *args, **kwargs):
    with RequestRetrySession() as sess:
        res = sess.request(method, *args, **kwargs)
        raise_for_status(res)
        res.encoding = "utf-8"
        return res.json()


def request_with_token(method, *args, **kwargs):
    if 'headers' not in kwargs:
        kwargs['headers'] = {}
    auth = configure_auth()
    kwargs['headers']['Authorization'] = f"Bearer {auth.access_token}"
    try:
        return request(method, *args, **kwargs)
    except HTTPError as e:
        if e.response.status_code == 401 and auth.refresh_token:
            auth = configure_auth(force=True)
            if auth:
                return request_with_token(method, *args, **kwargs)
        raise


def get_with_token(*args, **kwargs):
    return request_with_token('GET', *args, **kwargs)


def post_with_token(*args, **kwargs):
    return request_with_token('POST', *args, **kwargs)


def delete_with_token(*args, **kwargs):
    return request_with_token('DELETE', *args, **kwargs)


def put_with_token(*args, **kwargs):
    return request_with_token('PUT', *args, **kwargs)


def patch_with_token(*args, **kwargs):
    return request_with_token('PATCH', *args, **kwargs)



import os
import responses
from requests import RequestException

from anylearn.interfaces.resource.resource_downloader import SyncResourceDownloader
from tests.base_test_case import BaseTestCase

class TestResourceDownloader(BaseTestCase):

    @responses.activate
    def test_run_sync_resource_downloader_ok(self):
        responses.add(responses.POST, url=self._url("resource/compression"),
                        match_querystring=True,
                        json={'data': "COMP123"},
                        status=200)
        responses.add(responses.GET, url=self._url("resource/compression?file_id=FILE001&compression_id=COMP123"),
                            match_querystring=True,
                            json=[{'id': "COMP123", 'state': 2}],
                            status=200)
        downloader = SyncResourceDownloader()
        file_name = "test_sync_download.zip"
        headers = {'Content-Disposition': "attachment; filename=%s;" % file_name,}
        responses.add(responses.GET, url=self._url("resource/download?file_id=FILE001&compression_id=COMP123&token=TEST_TOKEN"),
                      match_querystring=True,
                      headers=headers,
                      status=200)
        res = downloader.run(resource_id="FILE001",
                             save_path="tests/")
        os.remove(f"tests/{file_name}")
        self.assertTrue(res)

    @responses.activate
    def test_run_sync_resource_downloader_500(self):
        downloader = SyncResourceDownloader()
        responses.add(responses.GET, url=self._url("resource/download?file_id=FILE001&token=TEST_TOKEN"),
                      status=500)
        with self.assertRaises(RequestException) as ctx:
            downloader.run(resource_id="FILE001",
                           save_path="/data/wtt")
        e = ctx.exception
        self.assertIsInstance(e, RequestException)



from __future__ import annotations

import hashlib
import os
import time
import uuid
from pathlib import Path
from typing import Optional, Tuple, Union

from rich import print

from anylearn.sdk.errors import (
    AnylearnInvalidResponseError,
    AnylearnArtifactTooLargeError,
)
from anylearn.sdk.client import (
    get_with_token,
    patch_with_token,
    post_with_token,
)
from anylearn.sdk.console import console_error, console_warning
from anylearn.sdk.context import get_base_url
from anylearn.sdk.sshkey import SSHKey, generate_ssh_key_pair
from anylearn.sdk.utils import get_config_path


def get_jumps_dir_path() -> Path:
    jumps_dir = get_config_path().parent / "jumps"
    jumps_dir.mkdir(parents=True, exist_ok=True)
    return jumps_dir


JUMPS_SSH_KEY_NAME = "jumpsk"
JUMPS_ARTIFACT_SIZE_SOFT_LIMIT = 20 * 1024 * 1024 * 1024 # 20GB
JUMPS_ARTIFACT_SIZE_HARD_LIMIT = 50 * 1024 * 1024 * 1024 # 50GB


def get_jumps_ssh_key_paths() -> Tuple[Path, Path]:
    root = get_jumps_dir_path()
    private_key_path = root / JUMPS_SSH_KEY_NAME
    public_key_path = private_key_path.with_suffix(".pub")
    return private_key_path, public_key_path


def load_jumps_ssh_key() -> Optional[SSHKey]:
    private_key_path, public_key_path = get_jumps_ssh_key_paths()
    if not private_key_path.exists() or not public_key_path.exists():
        return None
    return SSHKey(
        public_key=public_key_path.read_bytes(),
        private_key=private_key_path.read_bytes(),
    )


def dump_jumps_ssh_key(ssh_key: SSHKey) -> None:
    private_key_path, public_key_path = get_jumps_ssh_key_paths()
    private_key_path.write_bytes(ssh_key.private_key)
    private_key_path.chmod(0o600)
    public_key_path.write_bytes(ssh_key.public_key)


class JumpsChannel:

    def __init__(self, *_, **kwargs) -> None:
        self.id = kwargs.get('id', None)
        self.client_code = kwargs.get('client_code', None)
        self.ip = kwargs.get('ip', None)
        self.port = kwargs.get('port', None)
        self.username = kwargs.get('username', None)
        self.path = kwargs.get('path', None)

    @classmethod
    def from_artifact_id(self, artifact_id: str) -> JumpsChannel:
        url = f"{get_base_url()}/jumps/channels/asset/{artifact_id}"
        res = get_with_token(url)
        if not res or not isinstance(res, dict):
            raise AnylearnInvalidResponseError(
                "Failed to get channel on jump server."
            )
        return JumpsChannel(**res)

    @classmethod
    def create(
        cls,
        artifact_id: str,
        artifact_local_path: Optional[Union[str, bytes, os.PathLike]],
    ) -> JumpsChannel:
        # Prepare SSH key
        ssh_key = load_jumps_ssh_key()
        if ssh_key is None:
            ssh_key = generate_ssh_key_pair()
            dump_jumps_ssh_key(ssh_key)
        # Total number of files and total size
        artifact_local_path = Path(artifact_local_path)
        if not artifact_local_path.exists():
            raise FileNotFoundError(
                f"Local artifact path {artifact_local_path} not found"
            )
        # TODO: avoid blocking the main thread since artifact may be huge
        size, nfiles = cls.get_local_artifact_stat(artifact_local_path)
        # Client conformity identifier
        client_identifier = cls.get_client_identifier(artifact_local_path)
        # Create channel
        res = post_with_token(
            f"{get_base_url()}/jumps/channels",
            data={
                'asset_id': artifact_id,
                'ssh_public_key': ssh_key.public_key.decode(),
                'total_size': size,
                'nb_files': nfiles,
                'client_code': client_identifier,
            }
        )
        if not res or not isinstance(res, dict):
            raise AnylearnInvalidResponseError(
                "Failed to create channel on jump server."
            )
        return JumpsChannel(**res)

    @classmethod
    def get_local_artifact_stat(
        cls,
        artifact_local_path: Optional[Union[str, bytes, os.PathLike]],
    ) -> Tuple[int, int]:
        artifact_local_path = Path(artifact_local_path)
        if not artifact_local_path.exists():
            raise FileNotFoundError(
                f"Local artifact path {artifact_local_path} not found"
            )
        if artifact_local_path.is_file():
            size, nfiles = artifact_local_path.stat().st_size, 1
            cls.check_artifact_size(size)
        elif not artifact_local_path.is_dir():
            raise FileNotFoundError(
                f"Local artifact path {artifact_local_path} is "
                "neither a file nor a directory"
            )
        size = 0
        nfiles = 0
        for f in artifact_local_path.glob("**/*"):
            if f.is_file():
                size += f.stat().st_size
                nfiles += 1
                cls.check_artifact_size(size)
        return size, nfiles

    @classmethod
    def check_artifact_size(cls, size: int) -> None:
        if size > JUMPS_ARTIFACT_SIZE_HARD_LIMIT:
            console_error(
                "artifact size exceeds Anylearn's size limit of "
                "[bold]50GB[/bold] "
                "for upload via jump server."
            )
            raise AnylearnArtifactTooLargeError(
                "Artifact size must be less than 50GB"
            )
        elif size > JUMPS_ARTIFACT_SIZE_SOFT_LIMIT:
            console_warning(
                "artifact size is larger than "
                "Anylearn's recommended maximum size of "
                "[bold]20GB[/bold] "
                "for upload via jump server."
            )

    @classmethod
    def get_client_identifier(cls, artifact_local_path: Optional[Union[str, bytes, os.PathLike]]) -> str:
        artifact_local_path = Path(artifact_local_path)
        mac_address = hex(uuid.getnode())
        code = f"{str(artifact_local_path)}@{mac_address}"
        h = hashlib.blake2s()
        h.update(code.encode())
        return h.hexdigest()

    def transform(self) -> bool:
        # Create transformation
        url = f"{get_base_url()}/jumps/channels/{self.id}/trans"
        res = post_with_token(url)
        if not res or not isinstance(res, dict):
            raise AnylearnInvalidResponseError(
                "Failed to create transformation on jump server."
            )
        # Poll transformation status
        status = self.get_transformation_status()
        while not status or status == 'running':
            status = self.get_transformation_status()
            time.sleep(1)
        return status == "succeeded"

    def get_transformation_status(self) -> Optional[str]:
        url = f"{get_base_url()}/jumps/channels/{self.id}/trans"
        res = get_with_token(url)
        if not res or not isinstance(res, dict):
            raise AnylearnInvalidResponseError(
                "Failed to get transformation status."
            )
        return res.get('status')

    def finish(self):
        patch_with_token(f"{get_base_url()}/jumps/channels/{self.id}/finishing")

    def __repr__(self) -> str:
        return (
            "JumpsChannel("
            f"id={self.id}, "
            f"ip={self.ip}, "
            f"port={self.port}, "
            f"username={self.username}, "
            f"path={self.path}"
            ")"
        )



import typer

from anylearn.sdk.utils import DEFAULT_ANYLEARN_HOST


HostArgument = typer.Argument(
    default=DEFAULT_ANYLEARN_HOST,
    help="The url (scheme included) of Anylearn to connect with.",
    show_default=True,
)


HostOption = typer.Option(
    DEFAULT_ANYLEARN_HOST,
    "--host",
    "-h",
    help="The url (scheme included) of Anylearn to connect with.",
    show_default=True,
)



import copy
import json
import os
from pathlib import Path
from requests import HTTPError
from typing import Dict, List, Optional, Tuple, Union

from anylearn.sdk import (
    AlgorithmArtifact,
    Artifact,
    DatasetArtifact,
    ModelArtifact,
    Project as SDKProject,
)
from anylearn.utils.cmd import cmd_error, cmd_warning
from .algorithm_manager import sync_algorithm
from .utils import (
    generate_random_name,
    get_mirror_by_name,
)
from ..interfaces import (
    Project,
    QuotaGroup,
    TrainTask,
    TrainTag,
)
from ..interfaces.resource import (
    Algorithm,
    Dataset,
    Model,
    Resource,
    ResourceUploader,
)
from ..utils.errors import (
    AnyLearnException,
    AnylearnRequiredLocalCommitException,
)


def _get_or_create_default_project():
    try:
        return Project.get_my_default_project()
    except:
        return Project.create_my_default_project()


def _resource_request_check(quota_group_request: Dict[str, Dict[str, int]]):
    if not isinstance(quota_group_request, dict):
        raise AnyLearnException("Parameter 'quota_group_request' should be a dictionary")
    for key in quota_group_request:
        if key == 'name':
            if not isinstance(quota_group_request['name'], str):
                raise AnyLearnException("The value of the key 'name' in the dictionary 'quota_group_request' should be a string")
        else:
            if not isinstance(quota_group_request[key], int):
                raise AnyLearnException(f"The value of the key '{key}' in the dictionary 'quota_group_request' should be an integer")


def _format_resource_request(resource_request: List[Dict[str, Dict[str, int]]]):
    if not resource_request:
        return None
    # Resource group by ID or name
    for i, req in enumerate(copy.deepcopy(resource_request)):
        for key, val in req.items():
            if key in ['default', 'besteffort'] or key.startswith('QGRP'):
                # Already ID
                continue
            try:
                qid = QuotaGroup(name=key, load_detail=True).id
            except AnyLearnException:
                raise AnyLearnException(f"Failed to find QuotaGroup: {key}")
            resource_request[i][qid] = val
            del resource_request[i][key]
    return resource_request


def _create_train_task(name: str,
                       algorithm: Algorithm,
                       project: Project,
                       hyperparams: dict,
                       hyperparams_prefix: str="--",
                       hyperparams_delimeter: str=" ",
                       envs: Optional[Dict[str, str]]=None,
                       mounts: Optional[Union[Dict[str, List[Resource]], Dict[str, Resource]]]=None,
                       artifacts: Optional[List[Artifact]]=None,
                       algorithm_git_ref: Optional[str]=None,
                       resource_request: Optional[List[Dict[str, Dict[str, int]]]]=None,
                       description: Optional[str]=None,			
                       entrypoint: Optional[str]=None,
                       output: Optional[Union[os.PathLike, bytes, str]]=None,
                       algorithm_dir: Optional[Union[str, bytes, os.PathLike]]=None,
                       mirror_name: Optional[str]="QUICKSTART",
                       num_nodes: Optional[int]=1,
                       nproc_per_node: Optional[int]=1):
    resource_ids = set()
    train_params = hyperparams
    for k, v in mounts.items():
        if isinstance(v, list):
            resource_ids.update([v_item.id for v_item in v])
            train_params[k] = [f"\"${v_item.id}\"" for v_item in v]
        else:
            resource_ids.add(v.id)
            train_params[k] = f"\"${v.id}\""
    envs_json_str = json.dumps(envs) if envs else ""
    for artifact in artifacts:
        resource_ids.add(artifact.id)
    train_task = TrainTask(
        name=name,
        project_id=project.id,
        algorithm_id=algorithm.id,
        algorithm_git_ref=algorithm_git_ref,
        files=list(resource_ids),
        train_params=json.dumps(train_params),
        train_params_prefix=hyperparams_prefix,
        train_params_delimeter=hyperparams_delimeter,
        envs=envs_json_str,
        resource_request=resource_request,
        description=description,
        num_nodes=num_nodes,
        nproc_per_node=nproc_per_node,
        mirror_id=get_mirror_by_name(mirror_name).id,
    )
    if entrypoint is not None:
        train_task.entrypoint = entrypoint
    if output is not None:
        output_path = Path(output)
        output_path_ok = (
            not output_path.is_absolute()
            and '..' not in output_path.parts
        )
        if output_path_ok and algorithm_dir is not None:
            output_path_joined = algorithm_dir / output_path
            output_path_ok &= (
                not output_path_joined.exists()
                or (
                    not output_path_joined.is_symlink()
                    and output_path_joined.is_dir()
                    and len([*output_path_joined.iterdir()]) == 0
                )
            )
        if not output_path_ok:
            raise AnyLearnException(
                f'Invalid output path. A relative path without ".." required, '
                f'and that path must be pointing at nothing or an empty '
                f'directory (symlink not allowed). Got '
                f'"{output_path_ok}".'
            )
        train_task.output = output
    train_task.save()
    train_task.get_detail()
    return train_task


def upload(
    algorithm_cloud_name: str,
    algorithm_entrypoint: str=None,
    algorithm_output: Optional[Union[os.PathLike, bytes, str]]=None,
    algorithm_local_dir: Optional[Union[str, bytes, os.PathLike]]='.',
    algorithm_force_update: bool=False,
) -> Tuple[Algorithm, str]:
    try:
        return sync_algorithm(
            name=algorithm_cloud_name,
            dir_path=algorithm_local_dir,
            force=algorithm_force_update,
        )
    except AnylearnRequiredLocalCommitException:
        # Notify possible usage of algorithm_force_update=True
        raise AnylearnRequiredLocalCommitException(
            "Local algorithm code has uncommitted changes. "
            "Please commit your changes or "
            "specify `algorithm_force_update=True` "
            "to let Anylearn make an auto-commit."
    )


def run(
    algorithm_cloud_name: str,
    algorithm_entrypoint: str,
    algorithm_output: Optional[Union[os.PathLike, bytes, str]]='./output/',
    algorithm_hyperparams: dict={},
    algorithm_envs: Optional[Dict[str, str]]=None,
    dataset_cloud_names: Optional[List[str]]=None,
    model_cloud_names: Optional[List[str]]=None,
    project_name: Optional[str]=None,
    task_name: Optional[str]=None,
    image_name: Optional[str]="QUICKSTART",
    quota_group_request: Optional[Dict[str, Union[int, str]]]=None,
) -> Optional[TrainTask]:
    kw = locals()

    # Fetch algorithm by full name
    try:
        algo = AlgorithmArtifact.from_full_name(algorithm_cloud_name)
    except HTTPError as e:
        status_code = e.response.status_code
        if status_code == 404:
            cmd_error(f"Algorithm {algorithm_cloud_name} not found.")
        else:
            cmd_error(
                f"Error fetching algorithm {algorithm_cloud_name}, "
                "status code {status_code}."
            )
        return None
    kw['algorithm_id'] = algo.id

    # Fetch project by full name
    if project_name:
        try:
            project = SDKProject.from_full_name(kw.pop('project_name'))
        except HTTPError as e:
            status_code = e.response.status_code
            if status_code == 404:
                cmd_error(f"Project {project_name} not found.")
            else:
                cmd_error(
                    f"Error fetching project {project_name}, "
                    "status code {status_code}."
                )
            return None
        kw['project_id'] = project.id

    t, _, _, _ = quick_train(**kw)
    return t


def upload_and_run(
    algorithm_cloud_name: str,
    algorithm_entrypoint: str,
    algorithm_output: Optional[Union[os.PathLike, bytes, str]]='./output/',
    algorithm_hyperparams: dict={},
    algorithm_envs: Optional[Dict[str, str]]=None,
    algorithm_local_dir: Optional[Union[str, bytes, os.PathLike]]='.',
    algorithm_force_update: bool=False,
    dataset_cloud_names: Optional[List[str]]=None,
    model_cloud_names: Optional[List[str]]=None,
    project_name: Optional[str]=None,
    task_name: Optional[str]=None,
    image_name: Optional[str]="QUICKSTART",
    quota_group_request: Optional[Dict[str, Union[int, str]]]=None,
) -> TrainTask:
    kw = locals()

    # Fetch project by full name
    if project_name:
        try:
            project = SDKProject.from_full_name(kw.pop('project_name'))
        except HTTPError as e:
            status_code = e.response.status_code
            if status_code == 404:
                cmd_error(f"Project {project_name} not found.")
            else:
                cmd_error(
                    f"Error fetching project {project_name}, "
                    "status code {status_code}."
                )
            return None
        kw['project_id'] = project.id

    t, _, _, _ = quick_train(**kw)
    return t


def quick_train(
    algorithm_id: Optional[str] = None,
    algorithm_cloud_name: Optional[str] = None,
    algorithm_local_dir: Optional[Union[os.PathLike, bytes, str]] = None,
    algorithm_force_update: bool = False,
    algorithm_git_ref: Optional[str] = None,
    algorithm_entrypoint: Optional[str] = None,
    algorithm_output: Optional[Union[os.PathLike, bytes, str]] = "./output/",
    algorithm_hyperparams: Optional[Dict[str, str]] = None,
    algorithm_hyperparams_prefix: str = "--",
    algorithm_hyperparams_delimeter: str = " ",
    algorithm_envs: Optional[Dict[str, str]] = None,
    dataset_hyperparam_name: str = "dataset",
    dataset_id: Optional[Union[List[str], str]] = None,
    dataset_cloud_names: Optional[List[str]] = None,
    model_hyperparam_name: str = "model",
    model_id: Optional[Union[List[str], str]] = None,
    model_cloud_names: Optional[List[str]] = None,
    pretrain_hyperparam_name: str = "pretrain",
    pretrain_task_id: Optional[Union[List[str], str]] = None,
    project_id: Optional[str] = None,
    project_name: Optional[str] = None,
    tags: Optional[List[str]] = None,
    task_name: Optional[str] = None,
    task_description: Optional[str] = None,
    image_name: Optional[str] = "QUICKSTART",
    quota_group_request: Optional[Dict[str, Union[int, str]]] = None,
    num_nodes: Optional[int] = 1,
    nproc_per_node: Optional[int] = 1,
    _uploader: ResourceUploader = None,
    _polling: Union[float, int] = 0.5,
):
    """
    本地算法快速训练接口。

    仅需提供本地资源和训练相关的信息，
    即可在Anylearn后端引擎启动自定义算法/数据集的训练：
    
    - 算法路径（文件目录或压缩包）
    - 数据集路径（文件目录或压缩包）
    - 训练启动命令
    - 训练输出路径
    - 训练超参数

    本接口封装了Anylearn从零启动训练的一系列流程：

    - 算法注册、上传
    - 数据集注册、上传
    - 训练项目创建
    - 训练任务创建

    本地资源初次在Anylearn注册和上传时，
    会在本地记录资源的校验信息。
    下一次调用快速训练或快速验证接口时，
    如果提供了相同的资源信息，
    则不再重复注册和上传资源，
    自动复用远程资源。

    如有需要，也可向本接口传入已在Anylearn远程注册的算法或数据集的ID，
    省略资源创建的过程。

    Parameters
    ----------
    algorithm_id : :obj:`str`, optional
        已在Anylearn远程注册的算法ID。
    algorithm_cloud_name: :obj:`str`, optional
        指定的算法在Anylearn云环境中的名称。
        同一用户的自定义算法的名称不可重复。
        如有重复，则复用已存在的同名算法，
        算法文件将被覆盖并提升版本。
        原有版本仍可追溯。
    algorithm_local_dir : :obj:`str`, optional
        本地算法目录路径。
    algorithm_git_ref : :obj:`str`, optional
        算法Gitea代码仓库的版本号（可以是commit号、分支名、tag名）。
        使用本地算法时，如未提供此参数，则取本地算法当前分支名。
    algorithm_force_update : :obj:`bool`, optional
        在同步算法的过程中是否强制更新算法，如为True，Anylearn会对未提交的本地代码变更进行自动提交。默认为False。
    algorithm_entrypoint : :obj:`str`, optional
        启动训练的入口命令。
    algorithm_output : :obj:`str`, optional
        训练输出模型的相对路径（相对于算法目录）。
    algorithm_hyperparams : :obj:`dict`, optional
        训练超参数字典。
        超参数将作为训练启动命令的参数传入算法。
        超参数字典中的键应为长参数名，如 :obj:`--param` ，并省略 :obj:`--` 部分传入。
        如需要标识类参数（flag），可将参数的值设为空字符串，如 :obj:`{'my-flag': ''}` ，等价于 :obj:`--my-flag` 传入训练命令。
        默认为空字典。
    algorithm_hyperparams_prefix : :obj:`str`, optional
        训练超参数键前标识，可支持hydra特殊命令行传参格式的诸如 :obj:`+key1` 、 :obj:`++key2` 、 空前置 :obj:`key3` 等需求，
        默认为 :obj:`--` 。
    algorithm_hyperparams_delimeter :obj:`str`, optional
        训练超参数键值间的分隔符，默认为空格 :obj:` ` 。
    algorithm_envs : :obj:`dict`, optional
        训练环境变量字典。
    dataset_hyperparam_name : :obj:`str`, optional
        启动训练时，数据集路径作为启动命令参数传入算法的参数名。
        需指定长参数名，如 :obj:`--data` ，并省略 :obj:`--` 部分传入。
        数据集路径由Anylearn后端引擎管理。
        默认为 :obj:`dataset` 。
    dataset_id : :obj:`str`, optional
        已在Anylearn远程注册的数据集ID。
    dataset_cloud_names : :obj:`List[str]`, optional
        训练任务需使用的Anylearn云环境中的数据集的名称
    model_hyperparam_name : :obj:`str`, optional
        启动训练时，模型路径作为启动命令参数传入算法的参数名。
        需指定长参数名，如 :obj:`--model` ，并省略 :obj:`--` 部分传入。
        模型路径由Anylearn后端引擎管理。
        默认为 :obj:`model` 。
    model_id : :obj:`str`, optional
        已在Anylearn远程注册/转存的模型ID。
    model_cloud_names : :obj:`List[str]`, optional
        训练任务需使用的Anylearn云环境中的模型的名称
    pretrain_hyperparam_name: :obj:`str`, optional
        启动训练时，前置训练结果（间接抽象为“预训练”，即"pretrain"）路径作为启动命令参数传入算法的参数名。
        需指定长参数名，如 :obj:`--pretrain` ，并省略 :obj:`--` 部分传入。
        预训练结果路径由Anylearn后端引擎管理。
        默认为 :obj:`pretrain` 。
    pretrain_task_id: :obj:`List[str]` | :obj:`str`, optional
        在Anylearn进行过的训练的ID，一般为前缀TRAI的32位字符串。
        Anylearn会对指定的训练进行结果抽取并挂载到新一次的训练中。
    project_id : :obj:`str`, optional
        已在Anylearn远程创建的训练项目ID。
    tags : :obj:`list`, optional
        训练任务标签列表
    task_name : :obj:`str`, optional
        训练任务名称。
        若值为非空，则由SDK自动生成8位随机字符串作为训练任务名称。
    task_description : :obj:`str`, optional
        训练任务详细描述。
        若值为非空，
        且参数 :obj:`algorithm_force_update` 为 :obj:`True` 时，
        则Anylearn在自动提交本地算法变更时，
        会将此值作为commit message同步至远端
    image_name : :obj:`str`, optional
        训练使用的Anylearn云环境中的镜像的名称。
    quota_group_request : :obj:`dict`, optional
        训练所需计算资源组中资源数量。
        需指定 :obj:`key` 为 :obj:`name` , :obj:`value` 为 :obj:`资源组名称` ，若未指定则使用Anylearn后端的 :obj:`default` 资源组中的默认资源套餐。
    num_nodes : :obj:`int`, optional
        分布式训练需要的节点数。
    nproc_per_node : :obj:`int`, optional
        分布式训练每个节点运行的进程数。

    Returns
    -------
    TrainTask
        创建的训练任务对象
    Algorithm
        在快速训练过程中创建或获取的算法对象
    Dataset
        在快速训练过程中创建或获取的数据集对象
    Project
        创建的训练项目对象
    """
    # Resource request
    if quota_group_request is not None:
        _resource_request_check(quota_group_request)
        quota_group_name = (
            quota_group_request.pop('name')
            if 'name' in quota_group_request
            else 'default'
        )
        quota_group_request = [{quota_group_name: quota_group_request}]
    resource_request = _format_resource_request(quota_group_request)

    # Remote model
    if model_id is None:
        model_id = []
    elif not isinstance(model_id, list):
        model_id = [model_id]
    models = [
        Model(id=_id, load_detail=True)
        for _id in model_id
    ]

    # Remote dataset
    if dataset_id is None:
        dataset_id = []
    elif not isinstance(dataset_id, list):
        dataset_id = [dataset_id]
    datasets = [
        Dataset(id=_id, load_detail=True)
        for _id in dataset_id
    ]

    # Remote pretrain task results
    if pretrain_task_id is None:
        pretrain_task_id = []
    elif not isinstance(pretrain_task_id, list):
        pretrain_task_id = [pretrain_task_id]
    pretrain_tasks = [
        TrainTask(id=_id, load_detail=True)
        for _id in pretrain_task_id 
    ]

    artifacts: List[Artifact] = []
    # Remote dataset by full names
    if dataset_cloud_names:
        artifacts.extend([
            DatasetArtifact.from_full_name(_fn)
            for _fn in dataset_cloud_names
        ])
    # Remote model by full names
    if model_cloud_names:
        artifacts.extend([
            ModelArtifact.from_full_name(_fn)
            for _fn in model_cloud_names
        ])

    # Algorithm
    try:
        algo, current_sha = sync_algorithm(
            id=algorithm_id,
            name=algorithm_cloud_name,
            dir_path=algorithm_local_dir,
            mirror_name=image_name,
            force=algorithm_force_update,
            commit_msg=task_description,
            uploader=_uploader,
            polling=_polling,
        )
    except AnylearnRequiredLocalCommitException:
        # Notify possible usage of algorithm_force_update=True
        raise AnylearnRequiredLocalCommitException(
            "Local algorithm code has uncommitted changes. "
            "Please commit your changes or "
            "specify `algorithm_force_update=True` "
            "to let Anylearn make an auto-commit."
    )

    mounts = {}
    if datasets and dataset_hyperparam_name:
        mounts[dataset_hyperparam_name] = datasets
    if models and model_hyperparam_name:
        mounts[model_hyperparam_name] = models
    if pretrain_tasks and pretrain_hyperparam_name:
        mounts[pretrain_hyperparam_name] = [t.get_results_file() for t in pretrain_tasks]

    # Project
    if project_id:
        project = Project(id=project_id, load_detail=True)
    elif project_name:
        try:
            project = SDKProject.from_full_name(project_name)
            project = Project(id=project.id, load_detail=True) # TODO: unify tag in .sdk module then unify project models by SDKProject
        except HTTPError as e:
            status_code = e.response.status_code
            cmd_error(
                f"Error fetching project {project_name}, "
                f"status code {status_code}."
            )
            raise
    else:
        project = _get_or_create_default_project()

    # Train task
    train_task_name = task_name or generate_random_name()
    if algorithm_hyperparams is None:
        algorithm_hyperparams = {}
    train_task = _create_train_task(
        name=train_task_name,
        algorithm=algo,
        algorithm_git_ref=algorithm_git_ref or current_sha,
        project=project,
        hyperparams=algorithm_hyperparams,
        hyperparams_prefix=algorithm_hyperparams_prefix,
        hyperparams_delimeter=algorithm_hyperparams_delimeter,
        envs=algorithm_envs,
        mounts=mounts,
        artifacts=artifacts,
        resource_request=resource_request,
        description=task_description,
        num_nodes=num_nodes,
        nproc_per_node=nproc_per_node,
        entrypoint=algorithm_entrypoint,
        output=algorithm_output,
        algorithm_dir=algorithm_local_dir,
        mirror_name=image_name,
    )

    # Train tags
    if tags:
        train_tags = []
        project_tags = project.get_tags()
        for tag in list(set(tags)):
            flag = False
            for project_tag in project_tags:
                if tag == project_tag.name:
                    train_tags.append(project_tag)
                    flag = True
            if not flag:
                color, text_color = TrainTag.generate_random_tag_color()
                new_tag = TrainTag(
                    name=tag,
                    color=color,
                    text_color=text_color,
                    project_id=project.id,
                )
                new_tag.save()
                train_tags.append(new_tag)
        train_task.tag_tags(train_tags)

    return train_task, algo, datasets, project



import fnmatch
import hashlib
import os
import random
import re
import shutil
import string
from pathlib import Path
from typing import List, Optional, Union
from zipfile import ZipFile

from ..config import AnylearnConfig
from ..interfaces import Mirror
from ..utils import logger
from ..utils.errors import (
    AnyLearnMissingParamException,
    AnyLearnNotSupportedException,
)


def get_mirror_by_name(name: str) -> Mirror:
    mirrors = Mirror.get_list()
    try:
        return next(m for m in mirrors if m.name == name)
    except:
        raise AnyLearnNotSupportedException((
            f"Container for `{name}` is not supported by "
            "the connected backend."
        ))


def make_name_by_path(path: Union[str, Path]) -> str:
    if not path:
        raise AnyLearnMissingParamException("`path` required.")
    path = Path(path)
    basename = path.name
    suffix = hashlib.sha1(str(path).encode('utf-8')).hexdigest()[:8]
    return f"{basename}-{suffix}"


def generate_random_name() -> str:
    return ''.join(random.sample(string.ascii_lowercase + string.digits, 8))


def _check_resource_input(id: Optional[str]=None,
                          dir_path: Optional[Union[str, bytes, os.PathLike]]=None,
                          archive_path: Optional[Union[str, bytes, os.PathLike]]=None):
    if not any([id, dir_path, archive_path]):
        raise AnyLearnMissingParamException((
            "At least one of the parameters "
            "['id', 'dir_path', 'archive_path'] "
            "should be specified."
        ))


def _get_or_create_resource_archive(name,
                                    dir_path: Optional[Union[str, bytes, os.PathLike]]=None,
                                    archive_path: Optional[Union[str, bytes, os.PathLike]]=None):
    logger.info("Packaging resources...")
    if not archive_path or not Path(archive_path).exists():
        archive_path = shutil.make_archive(
            AnylearnConfig.workspace_path / name,
            "zip",
            dir_path
        )
    return archive_path


def _get_archive_checksum(archive_path: Optional[Union[str, bytes, os.PathLike]], buffer_size: int=65536):
    checksum = hashlib.blake2b()
    with open(archive_path, "rb") as f:
        while True:
            chunk = f.read(buffer_size)
            if not chunk:
                break
            checksum.update(chunk)
    return checksum.hexdigest()


def make_algorithm_zip(zip_filename: Optional[Union[os.PathLike, bytes, str]], algorithm_dir_path: Optional[Union[os.PathLike, bytes, str]]) -> Path:
    logger.info("Zipping algorithm files...")
    algorithm_files = get_algorithm_files(algorithm_dir_path)
    zip_path = Path(AnylearnConfig.workspace_path / zip_filename)
    with ZipFile(zip_path, "w") as zip_file:
        for file_path in algorithm_files:
            zip_file.write(file_path)
    return zip_path


def get_algorithm_files(algorithm_dir_path: Optional[Union[os.PathLike, bytes, str]]) -> List[str]:
    algorithm_dir_path = Path(algorithm_dir_path)
    if not algorithm_dir_path.exists():
        raise FileNotFoundError(
            f"Algorithm directory `{algorithm_dir_path}` does not exist."
        )

    gitignore_path = algorithm_dir_path / ".gitignore"
    if gitignore_path.exists():
        pattern = build_re_pattern_from_gitignore(gitignore_path)
    else:
        pattern = None

    algorithm_files = []
    for root, dirs, files in os.walk(algorithm_dir_path):
        if pattern:
            dirs[:] = [d for d in dirs if not re.match(pattern, d)]
            files[:] = [f for f in files if not re.match(pattern, f)]
        algorithm_files.extend([os.path.join(root, f) for f in files])
    return algorithm_files


def build_re_pattern_from_gitignore(gitignore_path: Optional[Union[os.PathLike, bytes, str]]) -> str:
    gitignore_path = Path(gitignore_path)
    if not gitignore_path.exists():
        raise FileNotFoundError(
            f"Gitignore file `{gitignore_path}` does not exist."
        )

    patterns = [".git"] # We ignore .git directory by default
    with gitignore_path.open("r") as f:
        for line in f:
            line = line.strip()
            if not line or line.startswith("#"):
                continue
            if line.endswith("/"):
                line = line[:-1]
            patterns.append(line)

    return r"|".join([fnmatch.translate(i) for i in patterns])



import os
from abc import ABC, abstractmethod
import cgi
import time
from typing import Optional, Union

import requests

from anylearn.config import AnylearnConfig
from anylearn.utils.errors import AnyLearnException
from anylearn.utils.api import get_with_token, post_with_token, url_base


class ResourceDownloader(ABC):
    """
    资源下载接口
    """
    @abstractmethod
    def run(self,
            resource_id: str,
            polling: Union[float, int],
            save_path: Optional[Union[str, bytes, os.PathLike]]=None,
           ):
        """
        执行资源下载,自定义资源上传需实现此方法

        Parameters
        ----------
        resource_id
            资源ID
        save_path
            保存路径
        """
        raise NotImplementedError


class SyncResourceDownloader(ResourceDownloader):
    """
    资源同步下载工具类
    """

    def run(self,
            resource_id: str,
            save_path: Optional[Union[str, bytes, os.PathLike]],
            polling: Union[float, int]=5,
           ):

        compress_res = _compress_file(resource_id=resource_id)
        print(f"Resource compress {compress_res}")

        compress_state_res = _compress_complete(resource_id=resource_id, compression_id=compress_res['data'])
        print(f"Resource compress state {compress_state_res}")

        while not compress_state_res[0]['state'] == 2:
            time.sleep(polling)
            compress_state_res = _compress_complete(resource_id=resource_id, compression_id=compress_res['data'])
            print(f"Resource compress state {compress_state_res}")

        headers = {'Authorization': f"Bearer {AnylearnConfig.token}"}

        res = requests.get(url=f"{url_base()}/resource/download",
                           headers=headers,
                           params={
                               'file_id': resource_id,
                               'compression_id': compress_res['data'],
                               'token': AnylearnConfig.token,
                           })
        res.raise_for_status()

        content_header = res.headers.get('Content-Disposition')
        if content_header:
            _, params = cgi.parse_header(content_header)
            fileName = params['filename']
            with open(f"{save_path}/{fileName}", 'wb') as f:
                f.write(res.content)
            return fileName
        else:
            return "文件下载失败"


def _compress_file(resource_id: str):
    res = post_with_token(f"{url_base()}/resource/compression",
                          data={'file_id': resource_id})
    if not res or 'data' not in res:
        raise AnyLearnException("请求未能得到有效响应")
    return res
    
def _compress_complete(resource_id: str, compression_id: str):
    res = get_with_token(f"{url_base()}/resource/compression",
                         params={
                             'file_id': resource_id,
                             'compression_id': compression_id,
                             })
    if not res or not isinstance(res, list):
        raise AnyLearnException("请求未能得到有效响应")
    return res



import typer
from rich import print

import anylearn.cli.jupload as jupload
from anylearn.cli._utils import HostArgument
from anylearn.sdk.auth import authenticate, disauthenticate
from anylearn.__about__ import __version__ as anylearn_version


app = typer.Typer(name="anyctl")
app.add_typer(jupload.app, name="jupload")


@app.command()
def login(host: str = HostArgument):
    if authenticate(host) is not None:
        print("[green]Login OK[/green]")
    else:
        print("[red]Login Failed[/red]")


@app.command()
def logout(host: str = HostArgument):
    disauthenticate(host)
    print("[green]Logout OK[/green]")


def version_callback(value: bool):
    if value:
        print(f"[green]{anylearn_version}")
        raise typer.Exit()


@app.callback()
def common(
    version: bool = typer.Option(
        None,
        "--version",
        "-v",
        help="Show the version and exit.",
        is_eager=True,
        callback=version_callback,
    ),
):
    pass



import io

import setuptools


SDK_INFO = {}
with open("./anylearn/__about__.py") as f:
    exec(f.read(), SDK_INFO)


try:
    with io.open("README.md", encoding="utf-8") as f:
        LONG_DESCRIPTION = f.read()
except FileNotFoundError:
    LONG_DESCRIPTION = ""

REQUIRES = []
with open('requirements.txt') as f:
    for line in f:
        line, _, _ = line.partition('#')
        line = line.strip()
        REQUIRES.append(line)

setuptools.setup(
    name=SDK_INFO['__package_name__'],
    version=SDK_INFO['__version__'],
    license=SDK_INFO['__license__'],
    description=SDK_INFO['__description__'],
    long_description=LONG_DESCRIPTION,
    long_description_content_type="text/markdown",
    author_email=SDK_INFO['__author_email__'],
    author=SDK_INFO['__author__'],
    install_requires=REQUIRES,
    python_requires='>=3.7',
    packages=setuptools.find_packages(),
    package_data={
        'anylearn': [
            "storage/db/migrations/alembic.ini",
        ],
    },
    include_package_data=True,
    classifiers=[
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        "Development Status :: 3 - Alpha",
        "Programming Language :: Python :: 3",
        "License :: Other/Proprietary License",
        "Operating System :: OS Independent",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Libraries",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    entry_points = '''
        [console_scripts]
        anyctl = anylearn.cli.cli:app
    ''',
)



from anylearn.sdk.utils import DEFAULT_ANYLEARN_HOST


class Context:
    def __init__(self, host: str = DEFAULT_ANYLEARN_HOST):
        self.host = host.rstrip("/")


__context__: Context = Context()


def get_context() -> Context:
    return __context__


def get_base_url() -> str:
    return f"{__context__.host}/api"


def init(host: str = DEFAULT_ANYLEARN_HOST) -> Context:
    global __context__
    __context__ = Context(host=host)
    return __context__



from __future__ import annotations

from anylearn.sdk.artifacts.artifact import Artifact
from anylearn.sdk.client import get_with_token
from anylearn.sdk.context import get_base_url
from anylearn.sdk.errors import AnylearnInvalidResponseError


class FileArtifact(Artifact):
    @classmethod
    def from_id(cls, id_: str) -> FileArtifact:
        res = get_with_token(
            f"{get_base_url()}/file/query",
            params={'id': id_},
        )
        if not res or not isinstance(res, list):
            raise AnylearnInvalidResponseError("Request failed")
        return FileArtifact(**res[0])
